package java.awt;

import java.awt.image.DataBuffer;
import sun.awt.image.SunWritableRaster;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.DataBufferInt;
import java.net.MalformedURLException;
import java.io.File;
import java.io.InputStream;
import java.net.URLConnection;
import java.io.IOException;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.util.logging.PlatformLogger;
import java.net.URL;
import java.awt.image.BufferedImage;

public final class SplashScreen
{
    private BufferedImage image;
    private final long splashPtr;
    private static boolean wasClosed;
    private URL imageURL;
    private static SplashScreen theInstance;
    private static final PlatformLogger log;
    
    SplashScreen(long splashPtr) {
        this.splashPtr = splashPtr;
    }
    
    public static SplashScreen getSplashScreen() {
        synchronized (SplashScreen.class) {
            if (GraphicsEnvironment.isHeadless()) {
                throw new HeadlessException();
            }
            if (!SplashScreen.wasClosed && SplashScreen.theInstance == null) {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        System.loadLibrary("splashscreen");
                        return null;
                    }
                });
                long getInstance = _getInstance();
                if (getInstance != 0L && _isVisible(getInstance)) {
                    SplashScreen.theInstance = new SplashScreen(getInstance);
                }
            }
            return SplashScreen.theInstance;
        }
    }
    
    public void setImageURL(URL imageURL) throws NullPointerException, IOException, IllegalStateException {
        this.checkVisible();
        URLConnection openConnection = imageURL.openConnection();
        openConnection.connect();
        int contentLength = openConnection.getContentLength();
        InputStream inputStream = openConnection.getInputStream();
        byte[] b = new byte[contentLength];
        int off = 0;
        while (true) {
            int available = inputStream.available();
            if (available <= 0) {
                available = 1;
            }
            if (off + available > contentLength) {
                contentLength = off * 2;
                if (off + available > contentLength) {
                    contentLength = available + off;
                }
                b = new byte[contentLength];
                System.arraycopy(b, 0, b, 0, off);
            }
            int read = inputStream.read(b, off, available);
            if (read < 0) {
                break;
            }
            off += read;
        }
        synchronized (SplashScreen.class) {
            this.checkVisible();
            if (!_setImageData(this.splashPtr, b)) {
                throw new IOException("Bad image format or i/o error when loading image");
            }
            this.imageURL = imageURL;
        }
    }
    
    private void checkVisible() {
        if (!this.isVisible()) {
            throw new IllegalStateException("no splash screen available");
        }
        return;
    }
    
    public URL getImageURL() throws IllegalStateException {
        synchronized (SplashScreen.class) {
            this.checkVisible();
            if (this.imageURL == null) {
                try {
                    String getImageFileName = _getImageFileName(this.splashPtr);
                    String getImageJarName = _getImageJarName(this.splashPtr);
                    if (getImageFileName != null) {
                        if (getImageJarName != null) {
                            this.imageURL = new URL("jar:" + new File(getImageJarName).toURL().toString() + "!/" + getImageFileName);
                        }
                        else {
                            this.imageURL = new File(getImageFileName).toURL();
                        }
                    }
                }
                catch (MalformedURLException t) {
                    if (SplashScreen.log.isLoggable(PlatformLogger.Level.FINE)) {
                        SplashScreen.log.fine("MalformedURLException caught in the getImageURL() method", t);
                    }
                }
            }
            return this.imageURL;
        }
    }
    
    public Rectangle getBounds() throws IllegalStateException {
        synchronized (SplashScreen.class) {
            this.checkVisible();
            float getScaleFactor = _getScaleFactor(this.splashPtr);
            Rectangle getBounds = _getBounds(this.splashPtr);
            assert getScaleFactor > 0.0f;
            if (getScaleFactor > 0.0f && getScaleFactor != 1.0f) {
                getBounds.setSize((int)(getBounds.getWidth() / (double)getScaleFactor), (int)(getBounds.getHeight() / (double)getScaleFactor));
            }
            return getBounds;
        }
    }
    
    public Dimension getSize() throws IllegalStateException {
        return this.getBounds().getSize();
    }
    
    public Graphics2D createGraphics() throws IllegalStateException {
        synchronized (SplashScreen.class) {
            this.checkVisible();
            if (this.image == null) {
                Dimension size = _getBounds(this.splashPtr).getSize();
                this.image = new BufferedImage(size.width, size.height, 2);
            }
            float getScaleFactor = _getScaleFactor(this.splashPtr);
            Graphics2D graphics = this.image.createGraphics();
            assert getScaleFactor > 0.0f;
            if (getScaleFactor <= 0.0f) {
                getScaleFactor = 1.0f;
            }
            graphics.scale((double)getScaleFactor, (double)getScaleFactor);
            return graphics;
        }
    }
    
    public void update() throws IllegalStateException {
        BufferedImage image;
        synchronized (SplashScreen.class) {
            this.checkVisible();
            image = this.image;
        }
        if (image == null) {
            throw new IllegalStateException("no overlay image available");
        }
        DataBuffer dataBuffer = image.getRaster().getDataBuffer();
        if (!(dataBuffer instanceof DataBufferInt)) {
            throw new AssertionError("Overlay image DataBuffer is of invalid type == " + dataBuffer.getClass().getName());
        }
        int numBanks = dataBuffer.getNumBanks();
        if (numBanks != 1) {
            throw new AssertionError("Invalid number of banks ==" + numBanks + " in overlay image DataBuffer");
        }
        if (!(image.getSampleModel() instanceof SinglePixelPackedSampleModel)) {
            throw new AssertionError("Overlay image has invalid sample model == " + image.getSampleModel().getClass().getName());
        }
        int scanlineStride = ((SinglePixelPackedSampleModel)image.getSampleModel()).getScanlineStride();
        Rectangle bounds = image.getRaster().getBounds();
        int[] stealData = SunWritableRaster.stealData((DataBufferInt)dataBuffer, 0);
        synchronized (SplashScreen.class) {
            this.checkVisible();
            _update(this.splashPtr, stealData, bounds.x, bounds.y, bounds.width, bounds.height, scanlineStride);
        }
        return;
    }
    
    public void close() throws IllegalStateException {
        synchronized (SplashScreen.class) {
            this.checkVisible();
            _close(this.splashPtr);
            this.image = null;
            markClosed();
        }
    }
    
    static void markClosed() {
        synchronized (SplashScreen.class) {
            SplashScreen.wasClosed = true;
            SplashScreen.theInstance = null;
        }
    }
    
    public boolean isVisible() {
        synchronized (SplashScreen.class) {
            return !SplashScreen.wasClosed && _isVisible(this.splashPtr);
        }
    }
    
    private static native void _update(long p0, int[] p1, int p2, int p3, int p4, int p5, int p6);
    
    private static native boolean _isVisible(long p0);
    
    private static native Rectangle _getBounds(long p0);
    
    private static native long _getInstance();
    
    private static native void _close(long p0);
    
    private static native String _getImageFileName(long p0);
    
    private static native String _getImageJarName(long p0);
    
    private static native boolean _setImageData(long p0, byte[] p1);
    
    private static native float _getScaleFactor(long p0);
    
    static {
        SplashScreen.wasClosed = false;
        SplashScreen.theInstance = null;
        log = PlatformLogger.getLogger("java.awt.SplashScreen");
    }
}
